@@ -1,14 +1,14 @@
 package Net::SFTP::Foreign::Common;
 
-our $VERSION = '0.01';
+our $VERSION = '1.48';
 
 use strict;
 use warnings;
 use Carp;
-use Scalar::Util qw(dualvar);
+use Scalar::Util qw(dualvar tainted);
 use Fcntl qw(S_ISLNK S_ISDIR);
 
-use Net::SFTP::Foreign::Helpers qw(_gen_wanted _ensure_list);
+use Net::SFTP::Foreign::Helpers qw(_gen_wanted _ensure_list _debug $debug);
 use Net::SFTP::Foreign::Constants qw(:status);
 
 my %status_str = ( SSH2_FX_OK, "OK",
@@ -21,11 +21,22 @@ my %status_str = ( SSH2_FX_OK, "OK",
 		   SSH2_FX_CONNECTION_LOST, "Connection lost",
 		   SSH2_FX_OP_UNSUPPORTED, "Operation unsupported" );
 
+our $debug;
+
 sub _set_status {
-    my ($sftp, $code, $str) = @_;
+    my $sftp = shift;
+    my $code = shift;
     if ($code) {
-	$str = $status_str{$code} unless defined $str;
-	$str = "Unknown status ($code)" unless defined $str;
+        my $str;
+        if (@_) {
+            $str = join ': ', @_;
+            ($str) = $str =~ /(.*)/
+                if (${^TAINT} && tainted $str);
+        }
+        unless (defined $str and length $str) {
+            $str = $status_str{$code} || "Unknown status ($code)";
+        }
+        $debug and $debug & 64 and _debug("_set_status code: $code, str: $str");
 	return $sftp->{_status} = dualvar($code, $str);
     }
     else {
@@ -36,11 +47,19 @@ sub _set_status {
 sub status { shift->{_status} }
 
 sub _set_error {
-    my ($sftp, $code, $str) = @_;
+    my $sftp = shift;
+    my $code = shift;
     if ($code) {
-	unless (defined $str) {
+        my $str;
+        if (@_) {
+            $str = join ': ', @_;
+            ($str) = $str =~ /(.*)/
+                if (${^TAINT} && tainted $str);
+        }
+        else {
 	    $str = $code ? "Unknown error $code" : "OK";
 	}
+        $debug and $debug & 64 and _debug("_set_err code: $code, str: $str");
 	return $sftp->{_error} = dualvar $code, $str;
     }
     else {
@@ -62,7 +81,7 @@ sub _set_errno {
 	if ($status == SSH2_FX_EOF) {
 	    return;
 	}
-	elsif ($status == SSH2_FX_NO_SUCH_FILE) {
+        elsif ($status == SSH2_FX_NO_SUCH_FILE) {
 	    $! = Errno::ENOENT();
 	}
 	elsif ($status == SSH2_FX_PERMISSION_DENIED) {
@@ -72,10 +91,10 @@ sub _set_errno {
 	    $! = Errno::EBADMSG();
 	}
 	elsif ($status == SSH2_FX_OP_UNSUPPORTED) {
-	    $! = Errnor::ENOTSUP()
+	    $! = Errno::ENOTSUP()
 	}
 	elsif ($status) {
-	    $! = Errnor::EIO()
+	    $! = Errno::EIO()
 	}
     }
 }
@@ -92,15 +111,20 @@ sub _call_on_error {
 # this method code is a little convoluted because we are trying to
 # keep in memory as few entries as possible!!!
 sub find {
-    my ($self, $dirs, %opts) = @_;
+    @_ >= 1 or croak 'Usage: $sftp->find($remote_dirs, %opts)';
+
+    my $self = shift;
+    my %opts = @_ & 1 ? ('dirs', @_) : @_;
 
     $self->_set_error;
     $self->_set_status;
 
+    my $dirs = delete $opts{dirs};
     my $follow_links = delete $opts{follow_links};
     my $on_error = delete $opts{on_error};
     my $realpath = delete $opts{realpath};
     my $ordered = delete $opts{ordered};
+    my $names_only = delete $opts{names_only};
     my $atomic_readdir = delete $opts{atomic_readdir};
     my $wanted = _gen_wanted( delete $opts{wanted},
 			      delete $opts{no_wanted} );
@@ -109,10 +133,12 @@ sub find {
 
     %opts and croak "invalid option(s) '".CORE::join("', '", keys %opts)."'";
 
+    $dirs = '.' unless defined $dirs;
+
     my $wantarray = wantarray;
     my (@res, $res);
     my %done;
-    my %rpdone;
+    my %rpdone; # used to detect cycles
 
     my @dirs = _ensure_list $dirs;
     my @queue = map { { filename => $_ } } ($ordered ? sort @dirs : @dirs);
@@ -124,26 +150,32 @@ sub find {
 	my $entry = shift;
 	my $fn = $entry->{filename};
 	for (1) {
-	    my $follow = $follow_links and S_ISLNK($entry->{a}->perm);
+	    my $follow = ($follow_links and S_ISLNK($entry->{a}->perm));
 
 	    if ($follow or $realpath) {
 		unless (defined $entry->{realpath}) {
-		    my $rp = $entry->{realpath} = $self->realpath($fn)
-			or next;
-		    $rpdone{$rp}++ and next;
+                    my $rp = $self->{realpath} = $self->realpath($fn);
+                    next unless (defined $rp and not $rpdone{$rp}++);
 		}
 	    }
 		
 	    if ($follow) {
-		$entry->{a} = $self->stat($fn)
-		    or next;
-		unshift @queue, $entry;
+                my $a = $self->stat($fn);
+                if (defined $a) {
+                    $entry->{a} = $a;
+                    # we queue it for reprocessing as it could be a directory
+                    unshift @queue, $entry;
+                }
 		next;
 	    }
 		
 	    if (!$wanted or $wanted->($self, $entry)) {
 		if ($wantarray) {
-		    push @res, $entry;
+                    push @res, ( $names_only
+                                 ? ( exists $entry->{realpath}
+                                     ? $entry->{realpath}
+                                     : $entry->{filename} )
+                                 : $entry )
 		}
 		else {
 		    $res++;
